<?php

require_once 'lib/tv/abstract_tv.php';
require_once 'lib/tv/default_epg_item.php';

class MainTv extends AbstractTv
{
    const BASE_EPG_URL = "http://s1.dune-club.info/plugins/globaltv/epg/";
    const CHANNEL_IDS_FILE = "channel_ids.txt";
    const CHANNEL_IDS_FILE_EEPG = "globaltv_channel_ids.txt";

    private $channel_epg_id_by_ch_id = null;
    private $channel_eepg_by_ch_id_and_day = array();
    private $opt_dune_params;

    private $sections;
    private $groups;
    private $group_ids;
    private $channels;
    private $channel_ids;
    private $all_channels_group;
    private $favorites_group;

    private $playback_urls_by_ch_id = array();

    private $country_code_uc = '';

    public function __construct()
    {
        $is_android = HD::is_android();
        $is_amlogic = false;
        if ($is_android)
        {
            $android_platform = trim(
                file_get_contents('/firmware/config/android_platform.txt'));
            $is_amlogic = 0 === strpos($android_platform, 's905');
        }
        $this->opt_dune_params = "";
        if ($is_android)
            $this->opt_dune_params = "engine:1";

        parent::__construct(true);
    }

    private static function get_dune_auth()
    {
        static $dune_auth = null;

        if (is_null($dune_auth))
        {
            $dune_auth = file_get_contents("/tmp/dune_auth.txt");
            hd_print("Dune auth: '$dune_auth'");
        }

        return $dune_auth;
    }

    public static function get_playback_url_cmd()
    {
        return MainHelper::BASE_EXT_API_URL . "cmd/get_playback_url";
    }

    public static function get_trial_status_cmd()
    {
        return MainHelper::BASE_EXT_API_URL . "cmd/get_trial_status";
    }

    private static function get_region()
    {
        $cfg = SystemConfig::get_instance();
        $cfg->ensure_loaded();

        $val = $cfg->widget_country ?
            $cfg->widget_country : $cfg->country_code_uc;
        $val = mb_strtoupper($val, 'UTF-8');
        if ($val == 'RUSSIAN FEDERATION' || $val == 'RUSSIA')
            $val = 'RU';
        hd_print("Effective region: $val");
        return $val;
    }

    public function get_playback_url_info($stream_id, $archive_ts, $duration_m)
    {
        $region = self::get_region();
        $dune_auth = self::get_dune_auth();
        $query = "stream_id=$stream_id&" .
            "region=$region&" .
            "$dune_auth";

        if ($archive_ts > 0 && $duration_m > 0)
        {
            $query = "$query&" .
                "archive_ts=$archive_ts&" .
                "duration_m=$duration_m";
        }

        $url = self::get_playback_url_cmd() . '?' . $query;
        try
        {
            $doc = HD::http_get_document($url);
        }
        catch (DuneCurlException $e)
        {
            hd_print("Can not get playback URL: network or http error");
            return array('status' => 'error', 'error_code' => 1, 'error' => 'Curl problem');
        }

        $obj = json_decode($doc);
        if (!is_object($obj) || !isset($obj->status))
        {
            hd_print("Can not get playback URL: invalid data loaded");
            return array('status' => 'error', 'error_code' => 1, 'error' => 'Invalid data from server');
        }

        if ($obj->status != "ok")
        {
            hd_print("Incorrect playback URL: '$doc'!");
            MainHelper::$cache->set_pf_playback_available(false);
            return array('status' => 'error', 'error_code' => $obj->error_code, 'error' => $obj->error);
        }

        return array('status' => 'ok', 'url' => $obj->url);
    }

    public function get_trial_status_info()
    {
        $dune_auth = self::get_dune_auth();
        $url = self::get_trial_status_cmd() . "?$dune_auth";
        try
        {
            $doc = HD::http_get_document($url);
        }
        catch (DuneCurlException $e)
        {
            hd_print("Can not get trial status!");
            return array('status' => 'error', 'error_code' => 1, 'error' => 'Curl problem');
        }

        $obj = json_decode($doc);
        if (!is_object($obj) || !isset($obj->status))
        {
            hd_print("Can not get playback URL: invalid data loaded");
            return array('status' => 'error', 'error_code' => 1, 'error' => 'Invalid data from server');
        }

        if ($obj->status != "ok")
        {
            hd_print("Incorrect trial status: '$doc'!");
            return array('status' => 'error', 'error_code' => $obj->error_code, 'error' => $obj->error);
        }

        return array('status' => 'ok', 'trial' => $obj->trial);
    }

    public function get_favorites_group() {
        return $this->favorites_group;
    }

    public function get_all_channels_group() {
        return $this->all_channels_group;
    }

    public function change_tv_favorites($fav_op_type, $ch_id, &$plugin_cookies)
    {
        hd_print("D change_tv_favorites($fav_op_type, $ch_id)");

        $action = parent::change_tv_favorites($fav_op_type, $ch_id, $plugin_cookies);
        if (!HD::with_rows_api())
            return $action;

        EpfsHandler::update_fav_epfs();
        return $this->invalidate_favorites(null);
    }

    public function invalidate_favorites($post_action)
    {
        $media_urls = array(
            EpfsHandler::get_epf_id(SectionsHelper::FAV_SECTION_ID),
            TvFavoritesScreen::get_media_url_str());
        return ActionFactory::invalidate_folders(
            $media_urls,
            ActionFactory::update_rows_menu($post_action));
    }

    public function change_tv_favorites_categories($fav_op_type, $category_id, &$plugin_cookies)
    {
        $action = parent::change_tv_favorites_categories($fav_op_type, $category_id, $plugin_cookies);
        if (!HD::with_rows_api())
            return $action;

        EpfsHandler::update_fav_epfs();
        return $this->invalidate_favorites(null);
    }

    ///////////////////////////////////////////////////////////////////////

    private function group_is_protected($group_id)
    {
        // XXX TODO
        return $group_id == '500' || 0 === strpos($group_id, '500_');
    }

    public function unload_channels()
    {
        hd_print("tv D: unload_channels");

        $this->sections = null;
        $this->all_channels_group = null;
        $this->favorites_group = null;
        $this->groups = null;
        $this->group_ids = null;
        $this->channels = null;
        $this->channel_ids = null;

        $this->channel_epg_id_by_ch_id = null;
        $this->channel_eepg_by_ch_id_and_day = array();

        $this->playback_urls_by_ch_id = array();
    }

    public function ensure_channels_loaded()
    {
        if (!isset($this->channels))
            $this->load_channels();
    }

    protected function load_channels()
    {
        hd_print("tv D: load_channels");

        $cache = MainHelper::$cache;
        $cache->ensure_cache_loaded();

        $this->channel_ids = $cache->get_channel_ids();
        $this->channels = $cache->get_channel_by_id_map();
        $this->group_ids = $cache->get_group_ids();
        $this->groups = $cache->get_group_by_id_map();
        $this->sections = $cache->get_section_by_id_map();

        $this->favorites_group = array(
            'id' => '__favorites',
            'name' => T::g('plugin_default_favorites'),
            'icon_url' => 'plugin_file://img/cat/fav.png',
            'panel_icon_url' => 'plugin_file://img/cat/fav.png',
        );

        $this->all_channels_group = array(
            'id' => "all",
            'name' => T::g('plugin_default_all_channels'),
            'icon_url' => 'plugin_file://img/cat/all.png',
            'panel_icon_url' => 'plugin_file://img/cat/all.png',
        );

        $this->process_loaded_channels();
    }

    private function remove_channels_and_groups_from_russia_section()
    {
        foreach ($this->sections['russia']['categories'] as $g_id)
        {
            $g = $this->groups[$g_id];
            foreach ($g['channels'] as $ch_id)
            {
                unset($this->channels[$ch_id]);
                MainHelper::remove_from_array_by_value($this->channel_ids, $ch_id);
            }

            unset($this->groups[$g_id]);
            MainHelper::remove_from_array_by_value($this->group_ids, $g_id);
            MainHelper::remove_from_array_by_value($this->sections['all']['categories'], $g_id);
        }
        $this->sections['russia']['categories'] = array();
    }

    private function process_loaded_channels()
    {
        $ch_count_by_g_id = array();
        foreach ($this->groups as $g_id => $g)
        {
            $icon_url = MainHelper::BASE_EXT_API_URL . "category_icons/${g_id}.png";
            $icon_url = HD::enable_caching($icon_url);
            $g['icon_url'] = $icon_url;
            $panel_icon_url = MainHelper::BASE_EXT_API_URL . "category_panel_icons/${g_id}.png";
            $panel_icon_url = HD::enable_caching($panel_icon_url);
            $g['panel_icon_url'] = $panel_icon_url;

            $base_name = preg_replace('/ \(.*\)$/', '', $g['name']);
            $c = count($g['channels']);
            $g['name'] = "$base_name ($c)";
            $this->groups[$g_id] = $g;
            $ch_count_by_g_id[$g_id] = $c;
        }

        foreach ($this->sections as $s_id => $s)
        {
            $sum = 0;
            foreach ($s['categories'] as $g_id)
                $sum += $ch_count_by_g_id[$g_id];

            $base_name = preg_replace('/ \(.*\)$/', '', $s['name']);
            $s['name'] = "$base_name ($sum)";
            $this->sections[$s_id] = $s;
        }
    }

    public function get_channel_panel_icon_url($c)
    {
        return MainHelper::$cache->get_channel_panel_icon_url($c);
    }

    public function get_channel_icon_url($c)
    {
        return MainHelper::$cache->get_channel_icon_url($c);
    }

    private static function read_cache_password()
    {
        $path = MainHelper::tv_password_path();
        return is_file($path) ? (string) file_get_contents($path) : '';
    }

    private static function write_cache_password($password)
    {
        $path = MainHelper::tv_password_path();
        if ($password == '')
            HD::safe_unlink($path);
        else
            file_put_contents($path, $password);
    }

    private function sync_xxx_cache()
    {
        $password = MainHelper::read_xxx_password();
        $cached_password = self::read_cache_password();
        if ($password == $cached_password)
            return;

        hd_print("tv I: clearing xxx/all due to pwd change ".
            "$cached_password -> $password");

        HD::safe_unlink(MainHelper::tv_cache_path('xxx'));
        HD::safe_unlink(MainHelper::tv_cache_path('all'));

        self::write_cache_password($password);
    }

    private function ensure_tv_cache($section_id, $with_eepg)
    {
        self::sync_xxx_cache();

        $cache = MainHelper::$cache;
        if ($cache->tv_cache_exist($section_id))
            return true;
        $sid = MainHelper::tv_cache_id($section_id);

        hd_print("Creating tv_cache for section: $sid");
        $ts1 = microtime(true);

        $channels = array();

        $section_groups = array();
        $all = $sid == 'all';
        if ($all) {
            foreach ($this->group_ids as $gid)
                $section_groups[$gid] = $this->groups[$gid];
        } else {
            $section = $this->sections[$section_id];
            foreach ($section['categories'] as $gid)
                $section_groups[$gid] = $this->groups[$gid];
        }

        foreach ($this->channel_ids as $cid)
        {
            $c = $this->channels[$cid];
            $gids = $c['group_ids'];

            $gid = $gids[1];
            if (!isset($section_groups[$gid]))
                continue;

            if ($all) {
                $group_ids = $gids;
            } else {
                $group_ids = array();
                foreach ($gids as $gid) {
                    if ($gid == 'all' || isset($section_groups[$gid]))
                        $group_ids[] = $gid;
                }
            }

            $ch = array(
                PluginTvChannel::id => $c['id'],
                PluginTvChannel::caption => $c['name'],
                PluginTvChannel::icon_url => $this->get_channel_panel_icon_url($c),
                PluginTvChannel::number => $c['lcn'],

                PluginTvChannel::group_ids => $group_ids,

                PluginTvChannel::future_epg_days => 4,

                PluginTvChannel::archive_past_sec => 14 * 86400,
                PluginTvChannel::archive_delay_sec => 0,

                PluginTvChannel::playback_url_is_stream_url => 1,
            );
            if (self::group_is_protected($gid))
                $ch[PluginTvChannel::is_protected] = 1;

            if (isset($c['archive_days']))
            {
                $archive_days = $c['archive_days'] - 1;
                if ($archive_days > 0) {
                    $ch[PluginTvChannel::past_epg_days] = $archive_days;
                    $ch[PluginTvChannel::have_archive] = true;
                }
            }

            if ($with_eepg && isset($c->epg_id))
            {
                $ch[PluginTvChannel::ext_epg_id] = $eepg_id;
                $ch[PluginTvChannel::ext_epg_enabled] = false; // TODO
            }

            $channels[] = $ch;
        }

        $groups = array();

        $g = $this->all_channels_group;
        $groups[] = array(
            PluginTvGroup::id => $g['id'],
            PluginTvGroup::caption => $g['name'],
            PluginTvGroup::icon_url => $g['panel_icon_url'],
//            PluginTvGroup::channel_ids => $all_channel_ids,
        );

        foreach ($section_groups as $g)
        {
            $groups[] = array(
                PluginTvGroup::id => $g['id'],
                PluginTvGroup::caption => $g['name'],
                PluginTvGroup::icon_url => HD::get($g, 'panel_icon_url', ''),
            );
        }

        $obj = array(
            PluginTvInfo::groups => $groups,
            PluginTvInfo::channels => $channels,
        );

        $res = $cache->write_tv_cache($section_id, $obj);

        $ts2 = microtime(true);
        hd_print(sprintf("tv D: cache created in %.2fs", $ts2 - $ts1));

        return $res;
    }

    public function get_tv_info(MediaURL $media_url, &$plugin_cookies)
    {
        $this->ensure_channels_loaded();

        $preview = isset($media_url->preview);

        $with_eepg = defined('PluginTvChannel::ext_epg_id') && 
            defined('PluginTvInfo::ext_epg_channel_ids_url') &&
            is_file("/firmware_ext/plugins/ext_epg/dune_plugin.xml");

        $section_id = HD::get($media_url, 'section_id', -1);

        if (!$preview)
        {
            if (!$this->ensure_tv_cache($section_id, $with_eepg))
                return null;
            $tv_cache_path = MainHelper::tv_cache_path($section_id);
        }

        $is_favorite_group = isset($media_url->is_favorites);
        $initial_group_id = strval($media_url->group_id);
        $initial_is_favorite = 0;

        if ($is_favorite_group)
        {
            $initial_group_id = null;
            $initial_is_favorite = 1;
        }

        $fav_channel_ids = $this->get_fav_channel_ids($plugin_cookies);

        $info = array
        (
            PluginTvInfo::show_group_channels_only => true,

            PluginTvInfo::favorites_supported => true,
            PluginTvInfo::favorites_icon_url => $this->favorites_group['icon_url'],

            PluginTvInfo::initial_channel_id => strval($media_url->channel_id),
            
            PluginTvInfo::initial_group_id => $initial_group_id,

            PluginTvInfo::initial_is_favorite => $initial_is_favorite,
            PluginTvInfo::favorite_channel_ids => $fav_channel_ids,
        );

        if (HD::is_android())
            $info[PluginTvInfo::icon_auto_reducing_disabled] = true;

        if (!$preview)
            $info[PluginTvInfo::channels_file] = $tv_cache_path;
        else
        {
            $groups = array();
            $channels = array();

            $g = $this->all_channels_group;
            $c = HD::get($this->channels, $media_url->channel_id);
            if (!$c)
                throw new Exception("Illegal state 023 ($media_url->channel_id)");

            $groups[] = array(
                PluginTvGroup::id => $g['id'],
                PluginTvGroup::caption => $g['name'],
                PluginTvGroup::icon_url => $g['panel_icon_url'],
            );

            $ch = array(
                PluginTvChannel::id => $c['id'],
                PluginTvChannel::caption => $c['name'],
                PluginTvChannel::number => $c['lcn'],
                PluginTvChannel::icon_url => $this->get_channel_panel_icon_url($c),
                PluginTvChannel::group_ids => array($g['id']),
                PluginTvChannel::playback_url_is_stream_url => 1,
            );
            $channels[] = $ch;

            $info[PluginTvInfo::groups] = $groups;
            $info[PluginTvInfo::channels] = $channels;
        }

        // vld
        if (isset($media_url->archive_tm)) {
            $info[PluginTvInfo::initial_archive_tm] =
                strval($media_url->archive_tm);
        }

        $info[PluginTvInfo::epg_font_size] = PLUGIN_FONT_SMALL;

        if (defined('PluginTvInfo::epg_text_size'))
        {
            $info[PluginTvInfo::epg_text_size] = 32;
            $info[PluginTvInfo::epg_page_size] = 14;

            $info[PluginTvInfo::groups_text_size] = 32;
            $info[PluginTvInfo::groups_page_size] = 14;

            $info[PluginTvInfo::channels_text_size] = 32;
            $info[PluginTvInfo::channels_page_size] = 14;
        }

        if ($with_eepg)
        {
            $info[PluginTvInfo::ext_epg_enabled] = false;
            $info[PluginTvInfo::ext_epg_channel_ids_url] = self::CHANNEL_IDS_FILE_EEPG;
        }

        if (HD::with_builtin_pcontrol())
        {
            $info[PluginTvInfo::builtin_pcontrol_enabled] = true;

            $password = MainHelper::read_xxx_password();
            if ($section_id == 'xxx' && $password != '')
                $info[PluginTvInfo::entered_pcode] = $password;
        }

        return $info;
    }

    public function get_tv_playback_url($ch_id,
        $archive_ts, $protect_code, &$plugin_cookies)
    {
        hd_print("get_tv_playback_url: ch_id $ch_id, ".
            "archive_ts $archive_ts, protect_code: " .
            (isset($protect_code) ? $protect_code : '<null>'));

        $ch = $this->get_channel($ch_id);

        $group_ids = $ch['group_ids'];
        $gid = $group_ids[1];
        if (!HD::with_builtin_pcontrol() && self::group_is_protected($gid))
        {
            $password = MainHelper::read_xxx_password();
            if ($password != '')
            {
                $id = $ch['id'];
                hd_print("xxx D: channel $id is protected (group $gid)");

                if ($protect_code != $password)
                {
                    if (isset($protect_code))
                    {
                        hd_print("xxx I: protect code is wrong ".
                            "'$protect_code' != '$password'");
                    }
                    return 'protected';
                }
            }
        }

        $this->ensure_channels_loaded();

        $cur_program_info = $this->get_program_info($ch_id, $archive_ts, $plugin_cookies);
        $dune_params = $this->opt_dune_params;

        $archive_ts = empty($archive_ts) ? -1 : $archive_ts;
        $duration_m = 0;
        if ($archive_ts > 0)
        {
            $len = 0;
            if (isset($cur_program_info) &&
                isset($cur_program_info->end_tm))
            {
                $time_to_end = $cur_program_info->end_tm - $archive_ts;
                $len = $time_to_end;
            }
            if ($len < 3600)
                $len = 3600;

            $duration_m = intval($len / 60);

            if ($dune_params != "")
                $dune_params .= ",";
            $dune_params .= "mediainfo:0";
        }

        $url = '';
        if (isset($this->playback_urls_by_ch_id[$ch_id]) && $archive_ts <= 0)
        {
            $url = $this->playback_urls_by_ch_id[$ch_id];
            hd_print("Use saved playback URL: '$url'");
        }
        else
        {
            $url_info = $this->get_playback_url_info($ch['stream_id'], $archive_ts, $duration_m);
            if ($url_info['status'] != 'ok')
            {
                hd_print("Error occured, code: " . $url_info['error_code'] . ", error : '" . $url_info['error'] . "'");
                return '';
            }

            $url = $url_info['url'];
        }

        // We reset all playback URLs cache after one use to prevent saving bad old URLs.
        $this->playback_urls_by_ch_id = array();

        $dp = "|||dune_params|||bufind_mode:1";
        if ($dune_params != "")
            $dp .= "," . $dune_params;

        $url = preg_replace('/^http:\/\/localhost:/', 'http://ts://localhost:',
            str_replace('//127.0.0.1:/', '//localhost:/', $url));
        hd_print("DEBUG url: $url");
        hd_print("DEBUG dune_params: $dp");
        return $url . $dp;
    }

    private function load_channel_ids_if_needed()
    {
        if (isset($this->channel_epg_id_by_ch_id))
            return;

        $this->ensure_channels_loaded();

        $channel_epg_id_by_stream_id = array();
        try
        {
            $channel_ids_url = self::BASE_EPG_URL . self::CHANNEL_IDS_FILE;
            $doc = HD::http_get_document($channel_ids_url);

            $separator = "\r\n";
            $line = strtok($doc, $separator);

            while ($line !== false) {
                preg_match('/([^ ]*) *= *([^ ]*)/', $line, $matches);
                if (count($matches) >= 3)
                {
                    $stream_id = $matches[1];
                    $epg_id = $matches[2];
                    $channel_epg_id_by_stream_id[$stream_id] = $epg_id;
                }

                $line = strtok($separator);
            }

            $channel_epg_id_by_ch_id = array();
            foreach ($this->channels as $ch_id => $ch)
            {
                if (!isset($ch['stream_id']))
                    continue;

                $stream_id = $ch['stream_id'];
                if (!isset($channel_epg_id_by_stream_id[$stream_id]))
                    continue;

                $epg_id = $channel_epg_id_by_stream_id[$stream_id];
                $channel_epg_id_by_ch_id[$ch_id] = $epg_id;
            }

            $this->channel_epg_id_by_ch_id = $channel_epg_id_by_ch_id;
        }
        catch (Exception $e)
        {
            hd_print("Exception at load_channel_ids_if_needed: " . print_r($e, true));
        }
    }

    private function try_get_day_eepg($ch_id, $day)
    {
        $this->load_channel_ids_if_needed();
        if (!isset($this->channel_epg_id_by_ch_id) ||
            !isset($this->channel_epg_id_by_ch_id[$ch_id]))
        {
            hd_print("Can not find $ch_id in channel_epg_id_by_ch_id");
            return null;
        }

        $epg_id = $this->channel_epg_id_by_ch_id[$ch_id];

        $epg_url = self::BASE_EPG_URL . "$epg_id/$day";
        try
        {
            $doc = HD::http_get_document($epg_url);
        }
        catch (DuneCurlException $e)
        {
            hd_print("Can not load EPG, ch_id: '$ch_id', day: '$day'");
            return null;
        }

        $obj = json_decode($doc);
        if (!$obj)
            throw new Exception("Failed to parse channel EPG");

        $eepg = array();
        foreach ($obj as $start_tm => $epg_info)
        {
            $len = 3600;
            if (isset($epg_info->len))
                $len = intval($epg_info->len);
            $epg_info->len = $len;

            $epg_info->end_tm = $start_tm + $len;

            $title = '';
            if (isset($epg_info->title))
                $title = strval($epg_info->title);
            $epg_info->title = $title;

            $ep_title = '';
            if (isset($epg_info->{'sub-title'}))
                $ep_title = strval($epg_info->{'sub-title'});
            if ($title != '' && $ep_title != '')
            {
                if (preg_match('/[.?!]$/', $title) == 0)
                    $title .= '.';
                if (preg_match('/[ \n]$/', $title) == 0)
                    $title .= ' ';
                $title .= $ep_title;
            }
            $epg_info->title_and_subtitle = $title;

            $desc = '';
            if (isset($epg_info->desc))
                $desc = strval($epg_info->desc);
            $epg_info->desc = $desc;

            $year = '';
            if (isset($epg_info->year))
                $year = strval($epg_info->year);
            $epg_info->year = $year;

            $eepg[$start_tm] = $epg_info;
        }

        return $eepg;
    }

    public function get_day_eepg($ch_id, $day)
    {
        if (isset($this->channel_eepg_by_ch_id_and_day[$ch_id]) &&
            isset($this->channel_eepg_by_ch_id_and_day[$ch_id][$day]))
            return $this->channel_eepg_by_ch_id_and_day[$ch_id][$day];

        $eepg = null;
        try
        {
            $eepg = $this->try_get_day_eepg($ch_id, $day);
        }
        catch (Exception $e)
        {
            hd_print("Exception at get_day_epg: $e");
        }

        if (!isset($eepg))
            $eepg = array();

        if (!isset($this->channel_eepg_by_ch_id_and_day[$ch_id]))
            $this->channel_eepg_by_ch_id_and_day[$ch_id] = array();

        $this->channel_eepg_by_ch_id_and_day[$ch_id][$day] = $eepg;

        return $eepg;
    }

    const DUMMY_LEN = 3600;

    private function get_dummy_eepg($day_start_ts, $day_end_ts)
    {
        $n = 1;
        $eepg = array();
        $start_tm = $day_start_ts;
        while ($start_tm < $day_end_ts)
        {
            $end_tm = $start_tm + self::DUMMY_LEN;
            $title = date('d.m.Y H:i', $start_tm + MainHelper::$timezone_diff_secs);

            $eepg[] = (object) array(
                'title' => $title,
                'title_and_subtitle' => $title,
                'desc' => '',
                'start_tm' => $start_tm,
                'end_tm' => $end_tm,
            );

            $n++;
            $start_tm = $end_tm;
        }

        return $eepg;
    }

    public function get_day_epg_iterator($ch_id, $day_start_ts, &$plugin_cookies)
    {
        $ch = $this->get_channel($ch_id);

        $day = date("Y-m-d", $day_start_ts);
        $day_end_ts = $day_start_ts + 86400;

        $ch = HD::get($this->channels, $ch_id);
        $archive_days = 0;
        if (isset($ch['archive_days']))
            $archive_days = $ch['archive_days'] - 1;

        $eepg = $this->get_day_eepg($ch_id, $day);
        if (count($eepg) == 0 && $archive_days > 0)
            $eepg = $this->get_dummy_eepg($day_start_ts, $day_end_ts);

        $epg = array();
        foreach ($eepg as $epg_info)
        {
            $epg_item = new DefaultEpgItem(
                $epg_info->title_and_subtitle,
                $epg_info->desc,
                $epg_info->start_tm,
                $epg_info->end_tm
            );
            $epg[] = $epg_item;
        }

        return new EpgIterator($epg, $day_start_ts, $day_end_ts);
    }

    public function get_program_info($ch_id, $timestamp, &$plugin_cookies)
    {
        $day = date("Y-m-d", $timestamp);

        $ch = $this->get_channel($ch_id);

        $eepg = $this->get_day_eepg($ch_id, $day);

        foreach ($eepg as $epg_info)
        {
            if ($timestamp >= $epg_info->start_tm &&
                $timestamp < $epg_info->end_tm)
                return $epg_info;
        }

        return null;
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_tv_stream_url($media_url, &$plugin_cookies)
    { throw new Exception('Error: get_tv_stream_url() is not supported.'); }

    public function get_day_epg($channel_id, $day_start_ts, &$plugin_cookies)
    {
        $epg = array();

        foreach ($this->get_day_epg_iterator($channel_id, $day_start_ts, $plugin_cookies) as $e)
        {
            $epg[] =
                array
                (
                    PluginTvEpgProgram::start_tm_sec => $e->get_start_time(),
                    PluginTvEpgProgram::end_tm_sec => $e->get_finish_time(),
                    PluginTvEpgProgram::name => $e->get_title(),
                    PluginTvEpgProgram::description => $e->get_description(),
                );
        }

        return $epg;
    }

    ///////////////////////////////////////////////////////////////////////

    public function get_group($group_id)
    {
        if (!isset($this->groups[$group_id]))
            throw new Exception("Unknown group: $group_id");
        return $this->groups[$group_id];
    }

    public function get_channel($channel_id)
    {
        if (!isset($this->channels[$channel_id]))
            throw new Exception("Unknown channel: $channel_id");
        return $this->channels[$channel_id];
    }

    public function channel_is_protected($channel_id)
    {
        if (!isset($this->channels[$channel_id]))
            return false;
        $channel = $this->channels[$channel_id];

        if (!isset($channel['group_ids']))
            return false;
        $gids = $channel['group_ids'];

        if (!isset($gids[1]))
            return false;
        $gid = $gids[1];

        return self::group_is_protected($gid);
    }

    public function get_channels()
    {
        return $this->channels;
    }

    public function get_channels_count($group_id)
    {
        if ($group_id == $this->favorites_group['id'])
        {
            $plugin_cookies = MainHelper::$plugin->get_plugin_cookies();
            return count($this->get_fav_channel_ids($plugin_cookies));
        }
        if ($group_id == $this->all_channels_group['id'])
            return count($this->channel_ids);

        $g = HD::get($this->groups, $group_id);
        if (!$g)
            return 0;
        return count($g['channels']);
    }

    public function get_sections()
    {
        return $this->sections;
    }

    public function get_groups()
    {
        return $this->groups;
    }

    public function get_group_ids()
    {
        return $this->group_ids;
    }

    public function get_group_ids_by_section_id($section_id, &$plugin_cookies)
    {
        if ($section_id == SectionsHelper::FAV_SECTION_ID)
            return $this->get_fav_category_ids($plugin_cookies);

        $section = HD::get($this->sections, $section_id);
        if ($section)
            return $section['categories'];

        hd_print("Main TV: section not found $section_id");
        return array();
    }

    public function get_group_by_id($id)
    {
        if (!isset($this->groups[$id]))
            return null;
        return $this->groups[$id];
    }

    public function get_channel_ids()
    {
        return $this->channel_ids;
    }

    public function get_channel_by_id($id)
    {
        if (!isset($this->channels[$id]))
            return null;
        return $this->channels[$id];
    }

    public function set_playback_url_for_ch_id($ch_id, $playback_url)
    {
        $this->playback_urls_by_ch_id[$ch_id] = $playback_url;
    }
}

?>
